package my.test.sakila.view;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.Stateful;
import javax.enterprise.context.Conversation;
import javax.enterprise.context.ConversationScoped;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.inject.Inject;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import my.test.sakila.model.FilmActor;
import my.test.sakila.model.Actor;
import my.test.sakila.model.Film;

/**
 * Backing bean for FilmActor entities.
 * <p>
 * This class provides CRUD functionality for all FilmActor entities. It focuses
 * purely on Java EE 6 standards (e.g. <tt>&#64;ConversationScoped</tt> for
 * state management, <tt>PersistenceContext</tt> for persistence,
 * <tt>CriteriaBuilder</tt> for searches) rather than introducing a CRUD framework or
 * custom base class.
 */

@Named
@Stateful
@ConversationScoped
public class FilmActorBean implements Serializable {

	private static final long serialVersionUID = 1L;

	/*
	 * Support creating and retrieving FilmActor entities
	 */

	private Long id;

	public Long getId() {
		return this.id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	private FilmActor filmActor;

	public FilmActor getFilmActor() {
		return this.filmActor;
	}

	@Inject
	private Conversation conversation;

	@PersistenceContext(type = PersistenceContextType.EXTENDED)
	private EntityManager entityManager;

	public String create() {

		this.conversation.begin();
		return "create?faces-redirect=true";
	}
	
	public void retrieve() {

		if (FacesContext.getCurrentInstance().isPostback()) {
			return;
		}

		if (this.conversation.isTransient()) {
			this.conversation.begin();
		}

		if (this.id == null) {
			this.filmActor = this.search;
		} else {
			this.filmActor = this.entityManager.find(FilmActor.class, getId());
		}
	}

	/*
	 * Support updating and deleting FilmActor entities
	 */

	public String update() {
		this.conversation.end();
		
		try {
			if (this.id == null) {
				this.entityManager.persist(this.filmActor);
				return "search?faces-redirect=true";			
			} else {
				this.entityManager.merge(this.filmActor);
				return "view?faces-redirect=true&id=" + this.filmActor.getId();
			}
		} catch( Exception e ) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage( e.getMessage() ));
			return null;
		}
	}

	public String delete() {
		this.conversation.end();

		try {
			this.entityManager.remove(this.entityManager.find(FilmActor.class,
					getId()));
			this.entityManager.flush();
			return "search?faces-redirect=true";
		} catch( Exception e ) {
			FacesContext.getCurrentInstance().addMessage(null, new FacesMessage( e.getMessage() ));
			return null;
		}
	}

	/*
	 * Support searching FilmActor entities with pagination
	 */

	private int page;
	private long count;
	private List<FilmActor> pageItems;
	
	private FilmActor search = new FilmActor();

	public int getPage() {
		return this.page;
	}

	public void setPage(int page) {
		this.page = page;
	}

	public int getPageSize() {
		return 10;
	}

	public FilmActor getSearch() {
		return this.search;
	}

	public void setSearch(FilmActor search) {
		this.search = search;
	}

	public void search() {
		this.page = 0;
	}

	public void paginate() {

		CriteriaBuilder builder = this.entityManager.getCriteriaBuilder();

		// Populate this.count

		CriteriaQuery<Long> countCriteria = builder.createQuery(Long.class);
		Root<FilmActor> root = countCriteria.from(FilmActor.class);
		countCriteria = countCriteria.select(builder.count(root)).where(
				getSearchPredicates(root));
		this.count = this.entityManager.createQuery(countCriteria)
				.getSingleResult();

		// Populate this.pageItems

		CriteriaQuery<FilmActor> criteria = builder.createQuery(FilmActor.class);
		root = criteria.from(FilmActor.class);
		TypedQuery<FilmActor> query = this.entityManager.createQuery(criteria
				.select(root).where(getSearchPredicates(root)));
		query.setFirstResult(this.page * getPageSize()).setMaxResults(
				getPageSize());
		this.pageItems = query.getResultList();
	}

	private Predicate[] getSearchPredicates(Root<FilmActor> root) {

		CriteriaBuilder builder = this.entityManager.getCriteriaBuilder();
		List<Predicate> predicatesList = new ArrayList<Predicate>();

		Actor actor = this.search.getActor();
		if (actor != null && actor.getActorId() != null) {
			predicatesList.add(builder.equal(root.get("actor"), actor));
		}
		Film film = this.search.getFilm();
		if (film != null && film.getFilmId() != null) {
			predicatesList.add(builder.equal(root.get("film"), film));
		}

		return predicatesList.toArray(new Predicate[predicatesList.size()]);
	}

	public List<FilmActor> getPageItems() {
		return this.pageItems;
	}

	public long getCount() {
		return this.count;
	}

	/*
	 * Support listing and POSTing back FilmActor entities (e.g. from inside an
	 * HtmlSelectOneMenu)
	 */

	public List<FilmActor> getAll() {

		CriteriaQuery<FilmActor> criteria = this.entityManager
				.getCriteriaBuilder().createQuery(FilmActor.class);
		return this.entityManager.createQuery(
				criteria.select(criteria.from(FilmActor.class))).getResultList();
	}

	public Converter getConverter() {

		return new Converter() {

			@Override
			public Object getAsObject(FacesContext context,
					UIComponent component, String value) {

				return FilmActorBean.this.entityManager.find(FilmActor.class,
						Long.valueOf(value));
			}

			@Override
			public String getAsString(FacesContext context,
					UIComponent component, Object value) {

				if (value == null) {
					return "";
				}

				return String.valueOf(((FilmActor) value).getId());
			}
		};
	}
	
	/*
	 * Support adding children to bidirectional, one-to-many tables
	 */
	 
	private FilmActor add = new FilmActor();

	public FilmActor getAdd() {
		return this.add;
	}

	public FilmActor getAdded() {
		FilmActor added = this.add;
		this.add = new FilmActor();
		return added;
	}
}